/**
 * @file LCD_mini_click.c
 * @brief <h2> LCD mini click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * LCD mini click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        14 Dec 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    ATMEGA32
 * -<b> Dev. Board    </b> :    EasyAVR v7
 * -<b> Ext. Modules  </b> :    LCD mini click
 * -<b> SW            </b> :    mikroC PRO for AVR v.6.1.2
 *
 */

/**
 * @page Dev. board setup
 *
 * 1) Put LCD mini click into mikroBUS 1 slot.
 *
 */


/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>

/******************************************************************************
* Preprocessors
*******************************************************************************/


/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/

// Port Expander module connections
sbit SPExpanderRST            at  PORTA6_bit;
sbit SPExpanderRST_Direction  at  DDA6_bit;
sbit SPExpanderCS             at  PORTA5_bit;
sbit SPExpanderCS_Direction   at  DDA5_bit;

sbit DigPot_CS            at  PORTA7_bit;
sbit DigPot_CS_Direction  at  DDA7_bit;

sbit PWM_DIR  at  DDD4_bit;

static uint8_t value = 0x00;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

static void system_init( void );
static void set_contrast( uint8_t value );
static void set_bcklight( uint8_t value );
/******************************************************************************
* Private Functions
*******************************************************************************/

/**************************************************************************//**
 *
 */
static void system_init( void )
{
    DigPot_CS = 1;
    DigPot_CS_Direction = 1;

    // Set PORTD pin 4 as output pin for the PWM.
    PWM_DIR = 1;

    // Initialise SPI.
    SPI1_Init_Advanced(
        _SPI_MASTER, 
        _SPI_FCY_DIV2,
        _SPI_CLK_LO_LEADING
    );

    // Initialise PWM for controlling back light.                                  
    PWM16bit_Init(
        _PWM16_PHASE_CORRECT_MODE_8BIT, 
        _PWM16_PRESCALER_16bit_8, 
        _PWM16_NON_INVERTED, 
        255, 
        1
    );  
    PWM16bit_Start(_TIMER1);
    PWM16bit_Change_Duty(0x00, _TIMER1_CH_B);

    SPI_Lcd_Config(0);
    SPI_Lcd_Cmd(_LCD_CLEAR);
    SPI_Lcd_Cmd(_LCD_CURSOR_OFF);
}

/**************************************************************************//**
 *
 */
static void set_contrast( uint8_t value )
{
    DigPot_CS = 0;

    SPI1_Write(0x00);
    SPI1_Write(~value);

    DigPot_CS = 1;
}

/**************************************************************************//**
 *
 */
static void set_bcklight( uint8_t value )
{
    PWM16bit_Change_Duty(value, _TIMER1_CH_B);
}

/******************************************************************************
* Public Functions
*******************************************************************************/

void main()
{
    system_init();
    
    SPI_Lcd_Out(1, 6, "mikroE");
    SPI_Lcd_Out(2, 2, "LCD mini click");

    set_bcklight(0xFF);
    set_contrast(0xDF);
    
    Delay_ms(5000);

    while (1)
    {
        set_bcklight(value);
        set_contrast(value);

        value++;
        delay_ms(40);
    }
}

/*************** END OF FUNCTIONS *********************************************/